#ifndef CLASS_H
#define CLASS_H

#include <frost_utils.h>

#include "typename.h"
#include "scope.h"
#include "templateparams.h"

namespace Frost
{

struct Variable;
struct Function;
struct Typedef;
struct Enum;
class  Compiler;

class Class
{
public :

    Class(s_ptr<Compiler> pCompiler, s_wptr<Module> pModule, const s_str& sType);

    void SetDocElements(const s_ctnr<s_str>& lDocElements);
    void SetType(const TypeName& mName);
    void SetScope(const s_str& sScope);
    s_wptr<Class> AddClass(const s_str& sName, const s_str& sType);
    void AddVariable(const Variable& mVariable);
    void AddFunction(const Function& mFunction);
    s_ptr<Typedef> AddTypedef(const Typedef& mTypedef);
    void AddEnum(const Enum& mEnum);
    void AddInheritance(const TypeName& mClass);
    void SetTemplateParams(const TemplateParams& mParams);
    s_ctnr<TypeName> DecorateType(const TypeName& mType, const s_ctnr<s_str>& lContext) const;

    s_ptr<Scope> GetLastScope() const;
    const TypeName& GetType() const;
    const s_str& GetClassType() const;
    const s_ctnr<Scope>& GetScopeList() const;
    const TemplateParams& GetTemplateParams() const;
    const s_ctnr<s_str>& GetDocElements() const;
    s_ctnr<s_ptr<const Class>> GetClassList() const;
    const s_bool& IsTemplate() const;
    s_bool InheritsFrom(s_wptr<Class> pClass) const;

    void CompileScopes(s_ctnr<s_str>& lContext);

private :

    TypeName       mType_;
    s_str          sClassType_;
    s_bool         bIsTemplate_;
    TemplateParams mTemplateParams_;
    s_ctnr<s_str>  lDocElements_;
    s_ctnr<Scope>  lScopeList_;
    s_ptr<Scope>   pCurrentScope_;

    s_ptr<Compiler> pCompiler_;
    s_wptr<Module>  pModule_;
};

}

#endif
