#ifndef COMMAND_H
#define COMMAND_H

#include <frost_utils.h>

namespace Frost
{

class Compiler;

class Command
{
public :

    Command(const s_str& sShortName, const s_str& sLongName, const s_str& sDescription);
    virtual ~Command();

    const s_str&  GetShortName() const;
    const s_str&  GetLongName() const;
    const s_str&  GetDescription() const;
    void          Disable();
    const s_bool& IsDisabled() const;
    const s_bool& IsProvided() const;

    void SetCompiler(s_ptr<Compiler> pCompiler);


    virtual void Read(s_ctnr<s_str>::const_iterator& iter, const s_ctnr<s_str>& lArgs) = 0;
    virtual void Do() = 0;

protected :

    s_str sShortName_;
    s_str sLongName_;
    s_str sDescription_;

    s_bool bDisabled_;
    s_bool bProvided_;

    s_ptr<Compiler> pCompiler_;
};

class CommandHelp : public Command
{
public :

    CommandHelp();

    void Read(s_ctnr<s_str>::const_iterator& iter, const s_ctnr<s_str>& lArgs);
    void Do();
};

class CommandNoCpp : public Command
{
public :

    CommandNoCpp();

    void Read(s_ctnr<s_str>::const_iterator& iter, const s_ctnr<s_str>& lArgs);
    void Do();

};

class CommandInput : public Command
{
public :

    CommandInput();

    void Read(s_ctnr<s_str>::const_iterator& iter, const s_ctnr<s_str>& lArgs);
    void Do();

private :

    s_str sInput_;
};

class CommandTarget : public Command
{
public :

    CommandTarget();

    void Read(s_ctnr<s_str>::const_iterator& iter, const s_ctnr<s_str>& lArgs);
    void Do();

private :

    s_str sTarget_;
};

class CommandVerbose : public Command
{
public :

    CommandVerbose();

    void Read(s_ctnr<s_str>::const_iterator& iter, const s_ctnr<s_str>& lArgs);
    void Do();
};

class CommandNoSafeWarning : public Command
{
public :

    CommandNoSafeWarning();

    void Read(s_ctnr<s_str>::const_iterator& iter, const s_ctnr<s_str>& lArgs);
    void Do();
};

class CommandOutput : public Command
{
public :

    CommandOutput();

    void Read(s_ctnr<s_str>::const_iterator& iter, const s_ctnr<s_str>& lArgs);
    void Do();

private :

    s_str sOutput_;
};

class CommandOutputDirectory : public Command
{
public :

    CommandOutputDirectory();

    void Read(s_ctnr<s_str>::const_iterator& iter, const s_ctnr<s_str>& lArgs);
    void Do();

private :

    s_str sOutputDirectory_;
};

class CommandCompiler : public Command
{
public :

    CommandCompiler();

    void Read(s_ctnr<s_str>::const_iterator& iter, const s_ctnr<s_str>& lArgs);
    void Do();

private :

    s_str sCompiler_;
};

class CommandCompilerCommands : public Command
{
public :

    CommandCompilerCommands();

    void Read(s_ctnr<s_str>::const_iterator& iter, const s_ctnr<s_str>& lArgs);
    void Do();

private :

    s_str sCompilerCommands_;
};

class CommandLinkerCommands : public Command
{
public :

    CommandLinkerCommands();

    void Read(s_ctnr<s_str>::const_iterator& iter, const s_ctnr<s_str>& lArgs);
    void Do();

private :

    s_str sLinkerCommands_;
};

class CommandException : public Exception
{
public :

    CommandException(s_ptr<Command> pCommand, const s_str& sError, const s_bool& bFatal=false);

    const s_bool& IsFatal() const;

private :

    s_bool bFatal_;
};

}

#endif // COMMAND_H
