#ifndef COMPILER_H
#define COMPILER_H

#include <frost_utils.h>
#include "templateparams.h"

namespace Frost
{

class  Command;
class  Module;
struct TypeName;
struct Function;
struct Variable;
struct Typedef;
struct Enum;

class  InputSource;

class Compiler
{
public :

    Compiler();
    ~Compiler();

    void ReadCommands(const s_ctnr<s_str>& lArgs);

    void RegisterCommand(s_refptr<Command> pCommand);
    s_wptr<Command> GetCommand(const s_str& sShortName);

    void PrintHelp() const;

    void SetCompiledFile(const s_str& sCompiled);
    void EnableVerboseMode();
    const s_bool& IsVerbose();
    void DisableSafeWarnings();
    void SetOutputFile(const s_str& sOutput);
    void SetOutputDirectory(const s_str& sOutputDirectory);
    const s_str& GetOutputDirectory() const;
    void DoNotCompileCpp();
    void SetCppCompiler(const s_str& sCompiler);
    void SetCppCompilerCommands(const s_str& sCompilerCommands);
    void SetCppLinkerCommands(const s_str& sLinkerCommands);
    void SetTarget(const s_str& sTarget);

    void Compile();
    void CompileProject(const s_str& sFile);
    s_wptr<Module> CompileModule(const s_str& sFile);
    s_wptr<Module> LoadModule(const s_str& sFile);
    void CompileCppFile(const s_str& sFile, const s_str& sOutput);
    void LinkFiles(const s_ctnr<s_str>& lFiles, const s_str& sOutput);

    void MakeDirectory(const s_str& sDirectory);

    s_bool FindModuleFile(s_str& sFileName);

    void ThrowError(const s_str& sError) const;
    void ThrowWarning(const s_str& sWarning, const s_bool& bIsSafe = false) const;

    void CheckOutputDirectory();

private :

    void Evaluate_(const s_ctnr<s_str>& lWords);
    s_wptr<Module> LoadModule_(const TypeName& mName);

    TypeName       ReadModuleName_(const s_ctnr<s_str>& lWords) const;
    void           ReadContent_(InputSource mInput);
    void           ReadClass_(const s_ctnr<s_str>& lWords, const s_bool& bIsTemplate = false);
    Function       ReadFunction_(const s_ctnr<s_str>& lWords, const s_bool& bIsTemplate = false) const;
    Variable       ReadVariable_(const s_ctnr<s_str>& lWords) const;
    Typedef        ReadTypedef_(const s_ctnr<s_str>& lWords) const;
    Enum           ReadEnum_(const s_ctnr<s_str>& lWords) const;
    TypeName       ReadType_(const s_ctnr<s_str>& lWords) const;
    TemplateParams ReadTemplateParams_(const s_ctnr<s_str>& lWords) const;
    void           CheckName_(const s_str& sName) const;
    s_bool         IsTemplateType_(const s_str& sType) const;

    struct Target
    {
        s_str sName_;
        s_str sCompiler_;
        s_str sCompilerCommands_;
        s_str sLinkerCommands_;
        s_str sOutputFile_;
        s_str sOutputDirectory_;
    };

    s_map<s_str, s_refptr<Command>> lCommandList_;
    s_bool bReady_;
    s_bool bVerbose_;
    s_bool bNoSafeWarning_;
    s_str  sOutputFile_;
    s_str  sOutputDirectory_;
    s_bool bOutputDirectoryChecked_;
    s_str  sCompiler_;
    s_str  sCompilerCommands_;
    s_str  sLinkerCommands_;
    s_str  sTargetName_;
    s_str  sDefaultTargetName_;
    s_map<s_str, Target> lTargetList_;

    s_str sCompiledFile_;
    s_str sCompiledProject_;
    s_str sProjectType_;
    s_bool bDoNotCompileCpp_;

    s_map<s_str, s_refptr<Module>> lModuleList_;
    s_map<s_str, s_refptr<Module>> lCompiledFileList_;

    struct State
    {
        s_wptr<Module> pCurrentModule_;
        s_str          sCurrentFile_;
        s_uint         uiCurrentLineNbr_;
        s_bool         bNestedClass_;
        s_bool         bUnion_;
        s_bool         bNoTypeCheck_;

        mutable s_ctnr<s_str>         lDocElements_;
        mutable TemplateParams        mTemplateParams_;
        mutable s_ctnr<s_ctnr<s_str>> lTemplateTypeStack_;
        s_ctnr<s_str>                 lContextStack_;
    };

    State mState_;

    s_ctnr<s_str> lRestrictedKeywordList_;
    s_ctnr<s_str> lPODTypeList_;
    s_ctnr<s_str> lTypeQualifierList_;
};

}

#endif // COMPILER_H
