#include <frost_utils.h>
#include <frost_utils_file.h>
#include <frost_utilsmanager.h>

#include <iostream>

#include "compiler.h"
#include "command.h"
#include "inputsource.h"

using namespace std;
using namespace Frost;

void LogFunction(const s_str& sMessage, const s_bool& bStamps, const s_uint& uiOffset)
{
    static File mLog("mcpp.log", File::O);

    s_str sNewMessage = sMessage;
    sNewMessage.Replace("\n", "\n"+s_str(' ', uiOffset));

    mLog.WriteLine(sNewMessage);
    mLog.Flush();
    cout << sMessage.Get() << endl;
}

int main(int argc, char* argv[])
{
    UtilsManager::GetSingleton()->SetLogFunction(&LogFunction);

    s_ctnr<s_str> lArgs;
    for (int i = 0; i < argc; ++i)
        lArgs.PushBack(argv[i]);

    try
    {
        Compiler mComp;
        mComp.RegisterCommand(s_refptr<Command>(new CommandHelp()));
        mComp.RegisterCommand(s_refptr<Command>(new CommandNoCpp()));
        mComp.RegisterCommand(s_refptr<Command>(new CommandInput()));
        mComp.RegisterCommand(s_refptr<Command>(new CommandTarget()));
        mComp.RegisterCommand(s_refptr<Command>(new CommandVerbose()));
        mComp.RegisterCommand(s_refptr<Command>(new CommandNoSafeWarning()));
        mComp.RegisterCommand(s_refptr<Command>(new CommandOutput()));
        mComp.RegisterCommand(s_refptr<Command>(new CommandOutputDirectory()));
        mComp.RegisterCommand(s_refptr<Command>(new CommandCompiler()));
        mComp.RegisterCommand(s_refptr<Command>(new CommandCompilerCommands()));
        mComp.RegisterCommand(s_refptr<Command>(new CommandLinkerCommands()));

        mComp.ReadCommands(lArgs);

        mComp.Compile();
    }
    catch (Exception& e)
    {
        Log(e.GetDescription());
        return 1;
    }
    catch (...)
    {
        Log("unhandled exception");
        return 1;
    }

    UtilsManager::Delete();

    return 0;
}
